"""LizyML: config-driven ML analysis library."""

from lizyml._version import __version__, __version_tuple__
from lizyml.core.model import Model

__all__ = ["Model", "__version__", "__version_tuple__"]
